@@ -103,9 +103,9 @@ struct Dictionary_s
 	const char * lang;
 	const char * version;
 	const char * locale;    /* Locale name */
-	double default_max_disjunct_cost;
 	locale_t     lctype;    /* Locale argument for the *_l() functions */
 	int          num_entries;
+	float default_max_disjunct_cost;
 	define_s     define;    /* Name-value definitions */
 
 	bool         use_unknown_word;
