@@ -48,6 +48,7 @@
  */
 
 #define D_EXPRUNE 9
+#define D_PRINT_NUM_DISJUNCTS 5
 
 #ifdef DEBUG
 #define DBG(p, w, X) \
@@ -194,8 +195,8 @@ static Exp* purge_Exp(exprune_context *ctxt, int, Exp *, char);
 static bool or_purge_operands(exprune_context *ctxt, int w, Exp *e, char dir)
 {
 #if NOTYET
-	const double nullexp_nonexistence = -9999;
-	double nullexp_mincost = nullexp_nonexistence;
+	const float nullexp_nonexistence = -9999;
+	float nullexp_mincost = nullexp_nonexistence;
 	int nullexp_count = 0;
 #endif
 
@@ -393,6 +394,23 @@ static char *print_expression_sizes(Sentence sent)
 	return dyn_str_take(e);
 }
 
+
+static void print_expression_disjunct_count(Sentence sent)
+{
+	uint64_t dcnt, t = 0;
+
+	for (WordIdx i = 0; i < sent->length; i++)
+	{
+		dcnt = 0;
+		for (const X_node *x = sent->word[i].x; x != NULL; x = x->next)
+			dcnt += count_clause(x->exp);
+		prt_error("%s(%zu) ", sent->word[i].alternatives[0], dcnt);
+		t += dcnt;
+	}
+	prt_error("\n\\");
+	prt_error("Total: %zu disjuncts\n\n", t);
+}
+
 void expression_prune(Sentence sent, Parse_Options opts)
 {
 	size_t w;
@@ -408,6 +426,12 @@ void expression_prune(Sentence sent, Parse_Options opts)
 
 	DBG_EXPSIZES("Initial expression sizes\n%s", e);
 
+	if (verbosity_level(D_PRINT_NUM_DISJUNCTS))
+	{
+		prt_error("Debug: Before expression_prune():\n\\");
+		print_expression_disjunct_count(sent);
+	}
+
 	for (int pass = 0; ; pass++)
 	{
 		/* Left-to-right pass */
@@ -485,6 +509,12 @@ void expression_prune(Sentence sent, Parse_Options opts)
 	}
 
 	free_connector_table(&ctxt);
+
+	if (verbosity_level(D_PRINT_NUM_DISJUNCTS))
+	{
+		prt_error("Debug: After expression_prune():\n\\");
+		print_expression_disjunct_count(sent);
+	}
 }
 
 #if 0 // VERY_DEAD_NO_GOOD_IDEA
