@@ -28,8 +28,7 @@
 #include <xlocale.h>
 #endif /* HAVE_LOCALE_T_IN_XLOCALE_H */
 
-#include "error.h"
-#include "lg_assert.h"
+#include "link-includes.h"
 
 #ifdef HAVE_ALLOCA_H
 # include <alloca.h>
@@ -58,19 +57,6 @@ void *alloca (size_t);
 #endif /* _MSC_VER */
 #endif /* !TLS */
 
-/* Windows, POSIX and GNU have different ideas about thread-safe strerror(). */
-#ifdef _WIN32
-#define lg_strerror_r(errno, buf, len) strerror_s(buf, len, errno)
-#else
-#if STRERROR_R_CHAR_P /* Set by "configure". */
-/* Emulate the POSIX version; assuming len>0 and a successful call. */
-#define lg_strerror_r(errno, buf, len) \
-	abs((strncpy(buf, strerror_r(errno, buf, len), len), buf[len-1] = '\0', 0))
-#else
-#define lg_strerror_r strerror_r
-#endif /* STRERROR_R_CHAR_P */
-#endif /* _WIN32 */
-
 #ifdef _MSC_VER
 /* These definitions are incorrect, as these functions are different(!)
  * (non-standard functionality).
@@ -98,14 +84,24 @@ void *alloca (size_t);
 #define HAVE_LOCALE_T 1
 #endif /* HAVE_LOCALE_T_IN_LOCALE_H || HAVE_LOCALE_T_IN_XLOCALE_H) */
 
+#if defined _MSC_VER || defined __cplusplus
+/* "restrict" is not a part of ISO C++.
+ * C++ compilers usually know it as __restrict. */
+#define restrict __restrict
+#endif
+
 #ifdef _WIN32
 #include <windows.h>
 #include <mbctype.h>
 
 /* Compatibility definitions. */
-#ifndef strncasecmp
-#define strncasecmp(a,b,s) strnicmp((a),(b),(s))
+#ifndef strcasecmp
+#define strcasecmp _stricmp
 #endif
+#ifndef strncasecmp
+#define strncasecmp _strnicmp
+#endif
+
 #undef rand_r  /* Avoid (a bad) definition on MinGW */
 int rand_r(unsigned int *);
 #ifndef __MINGW32__
@@ -158,6 +154,8 @@ size_t lg_mbrtowc(wchar_t *, const char *, size_t n, mbstate_t *ps);
  * are in the range [0..127], just limit its arguments to 7 bits. */
 #define lg_isspace(c) ((0 < c) && (c < 127) && isspace(c))
 
+void lg_strerror(int err_no, char *buf, size_t len);
+
 #if defined(__sun__)
 int strncasecmp(const char *s1, const char *s2, size_t n);
 /* This does not appear to be in string.h header file in sunos
@@ -168,7 +166,7 @@ int strncasecmp(const char *s1, const char *s2, size_t n);
 #ifdef HAVE_LOCALE_T
 locale_t newlocale_LC_CTYPE(const char *);
 #else
-typedef int locale_t;
+typedef void *locale_t;
 #define iswupper_l(c, l) iswupper(c)
 #define iswalpha_l(c, l) iswalpha(c)
 #define iswdigit_l(c, l) iswdigit(c)
@@ -220,7 +218,7 @@ void *aligned_alloc(size_t alignment, size_t size);
 #define strndupa(s, n) _strndupa3(alloca((n)+1), s, n)
 static inline char *_strndupa3(char *new_s, const char *s, size_t n)
 {
-	strncpy(new_s, s, n);
+	memcpy(new_s, s, n);
 	new_s[n] = '\0';
 
 	return new_s;
@@ -258,8 +256,9 @@ static inline char *_strndupa3(char *new_s, const char *s, size_t n)
 #if __GNUC__
 #define GCC_DIAGNOSTIC
 #define UNREACHABLE(x) (__extension__ ({if (x) __builtin_unreachable();}))
-#define GNUC_MALLOC __attribute__ ((malloc))
-#define GNUC_UNUSED __attribute__ ((unused))
+#define GNUC_MALLOC __attribute__ ((__malloc__))
+#define GNUC_UNUSED __attribute__ ((__unused__))
+#define GNUC_NORETURN __attribute__ ((__noreturn__))
 #define NO_SAN __attribute__ ((no_sanitize_address, no_sanitize_undefined))
 
 /* Define when configuring with ASAN/UBSAN - for fast dict load (of course
@@ -271,11 +270,20 @@ static inline char *_strndupa3(char *new_s, const char *s, size_t n)
 #define NO_SAN_DICT
 #endif
 
+#ifndef DONT_EXPECT
+#define likely(x)      __builtin_expect(!!(x), 1)
+#define unlikely(x)    __builtin_expect(!!(x), 0)
+#endif
+
 #else
 #define UNREACHABLE(x)
 #define GNUC_MALLOC
 #define GNUC_UNUSED
+#define GNUC_NORETURN
 #define NO_SAN_DICT
+
+#define likely(x) x
+#define unlikely(x) x
 #endif
 
 
@@ -467,7 +475,7 @@ void downcase_utf8_str(char *to, const char * from, size_t usize, locale_t);
 void upcase_utf8_str(char *to, const char * from, size_t usize, locale_t);
 #endif
 
-size_t lg_strlcpy(char * dest, const char *src, size_t size);
+size_t lg_strlcpy(char * restrict dst, const char * restrict src, size_t dsize);
 void safe_strcat(char *u, const char *v, size_t usize);
 char *safe_strdup(const char *u);
 
@@ -485,6 +493,7 @@ void dyn_strcat(dyn_str*, const char*);
 void dyn_trimback(dyn_str*);
 char * dyn_str_take(dyn_str*);
 const char * dyn_str_value(dyn_str*);
+size_t dyn_strlen(dyn_str*);
 
 size_t altlen(const char **);
 
@@ -509,7 +518,7 @@ size_t get_max_space_used(void);
 char * get_default_locale(void);
 void set_utf8_program_locale(void);
 bool try_locale(const char *);
-bool strtodC(const char *, double *);
+bool strtodC(const char *, float *);
 
 /**
  * Returns the smallest power of two that is at least i and at least 1
